﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using PageStateRecord = System.Tuple<object, object, System.DateTime?>;
using PageStateMap = System.Collections.Generic.Dictionary<string, System.Tuple<object, object, System.DateTime?>>;

/// <summary>
///     Stores ASP.NET page view state in the ASP.NET Session.
/// </summary>
public sealed class PageStateToSessionPersister : PageStatePersister
{
    #region Constants

    private const string c_keyPrefix = "QIS_PageState";
    private const string c_viewStateKey = "__PAGESTATEKEY";

    #endregion

    #region Fields

    private string m_pageStateKey;

    #endregion

    #region Constructors and Destructors

    /// <summary>
    ///     Initializes a new instance of the <see cref="PageStateToSessionPersister" /> class.
    /// </summary>
    /// <param name="page">
    ///     The <see cref="QPage" /> that the view state persistence mechanism is created for.
    /// </param>
    public PageStateToSessionPersister(Page page)
        : base(page)
    {
        m_pageStateKey = string.Format("{0}_{1}", c_keyPrefix, Guid.NewGuid());
    }

    #endregion

    #region Private Properties

    private HttpSessionState Session
    {
        [DebuggerNonUserCode]
        get
        {
            return Page.Session;
        }
    }

    private HttpApplicationState Application
    {
        [DebuggerNonUserCode]
        get
        {
            return Page.Application;
        }
    }

    #endregion

    #region Public Properties

    /// <summary>
    ///     Gets <see cref="System.String" /> value containing identifier of hidden field
    ///     that contains page stae key.
    /// </summary>
    public static string PageStateKey
    {
        get
        {
            return c_viewStateKey;
        }
    }

    #endregion

    #region Public Methods

    public void ClearPageState(string pageStateKey)
    {
        var pageStateMap = (PageStateMap)Application[c_viewStateKey];
        if (pageStateMap != null)
        {
            pageStateMap.Remove(pageStateKey);
        }
    }

    /// <summary>
    ///     Loads persisted state from the ASP.NET Session when a Page object initializes its control hierarchy.
    /// </summary>
    public override void Load()
    {
        File.AppendAllText("D:/Log.txt", "Load: " + Session.SessionID + Environment.NewLine);
        if (!Page.IsPostBack)
        {
            return;
        }

        m_pageStateKey = Page.Request.Form[c_viewStateKey];
        if (string.IsNullOrEmpty(m_pageStateKey) || !m_pageStateKey.StartsWith(c_keyPrefix))
        {
            throw new LoadViewStateFromSessionException(
                "Failed to load ViewState from ASP.NET Session: invalid key.");
        }

        var pageStateMap = (PageStateMap)Application[c_viewStateKey];
        PageStateRecord pageState;
        if (pageStateMap == null || !pageStateMap.TryGetValue(m_pageStateKey, out pageState))
        {
            File.AppendAllText("D:/Log.txt", "Load (null): " + Session.SessionID + Environment.NewLine);
            return;
            //throw new LoadViewStateFromSessionException(
                //"Failed to load ViewState from ASP.NET Session: invalid data.");
        }

        ViewState = pageState.Item1;
        ControlState = pageState.Item2;
    }

    /// <summary>
    ///     Writes any object state contained in the ViewState or in the ControlState property
    ///     to the ASP.NET Session.
    /// </summary>
    public override void Save()
    {
        File.AppendAllText("D:/Log.txt", "Save: " + Session.SessionID + Environment.NewLine);
        ScriptManager.RegisterHiddenField(Page, c_viewStateKey, m_pageStateKey);

        var pageStateMap = (PageStateMap)Application[c_viewStateKey];
        if (pageStateMap == null)
        {
            pageStateMap = new PageStateMap();
            Application[c_viewStateKey] = pageStateMap;
        }
        pageStateMap[m_pageStateKey] = Tuple.Create(ViewState, ControlState, (DateTime?)null);
    }

    #endregion
}

/// <summary>
///     Represents the exception that is thrown when ViewState cannot be loaded from ASP.NET Session.
/// </summary>
[Serializable]
public sealed class LoadViewStateFromSessionException : Exception
{
    #region Constructors and Destructors

    /// <summary>
    ///     Initializes a new instance of the <see cref="LoadViewStateFromSessionException" /> class
    ///     with a specified error message.
    /// </summary>
    /// <param name="message">
    ///     The <see cref="System.String" /> containing message that describes the error.
    /// </param>
    public LoadViewStateFromSessionException(string message)
        : base(message)
    {
        // Nothing to do
    }

    #endregion
}